#include <Arduino.h>
#include <Wire.h>

// XL9555 I2C地址（通常是0x20，但可能需要根据实际情况调整）
#define XL9555_ADDR 0x20

// XL9555寄存器地址
#define XL9555_INPUT_PORT_0  0x00
#define XL9555_OUTPUT_PORT_0 0x02
#define XL9555_CONFIG_PORT_0 0x06

// 定义编码器引脚
#define ENCODER1_A 6   // GPIO6
#define ENCODER1_B 7   // GPIO7
#define ENCODER2_A 15  // GPIO15
#define ENCODER2_B 16  // GPIO16

// 定义PWM引脚
#define PWMA 4  // GPIO4
#define PWMB 5  // GPIO5

// 定义I2C引脚
#define I2C_SCL 38  // GPIO38
#define I2C_SDA 39  // GPIO39
#define I2C_INT 8   // GPIO8 (如果需要中断功能)

// 定义PWM通道
#define PWM_CHANNEL_A 0
#define PWM_CHANNEL_B 1

// 定义PWM属性
#define PWM_FREQ 5000
#define PWM_RESOLUTION 8

// 定义测速相关变量
volatile long encoderValue1 = 0;
volatile long encoderValue2 = 0;
unsigned long lastTime = 0;
long lastPosition1 = 0;
long lastPosition2 = 0;
float rpm1 = 0;
float rpm2 = 0;

// 定义编码器每转的脉冲数（根据您的编码器规格进行调整）
const int PULSES_PER_REVOLUTION = 360;

// 中断服务函数
void IRAM_ATTR encoderISR1() {
  if (digitalRead(ENCODER1_B) == HIGH) {
    encoderValue1++;
  } else {
    encoderValue1--;
  }
}

void IRAM_ATTR encoderISR2() {
  if (digitalRead(ENCODER2_B) == HIGH) {
    encoderValue2++;
  } else {
    encoderValue2--;
  }
}

void setup() {
  Serial.begin(115200);
  
  // 设置编码器引脚
  pinMode(ENCODER1_A, INPUT_PULLUP);
  pinMode(ENCODER1_B, INPUT_PULLUP);
  pinMode(ENCODER2_A, INPUT_PULLUP);
  pinMode(ENCODER2_B, INPUT_PULLUP);
  
  // 设置PWM
  ledcSetup(PWM_CHANNEL_A, PWM_FREQ, PWM_RESOLUTION);
  ledcSetup(PWM_CHANNEL_B, PWM_FREQ, PWM_RESOLUTION);
  ledcAttachPin(PWMA, PWM_CHANNEL_A);
  ledcAttachPin(PWMB, PWM_CHANNEL_B);
  
  // 初始化I2C
  Wire.begin(I2C_SDA, I2C_SCL);
  
  // 配置XL9555 GPIO为输出
  Wire.beginTransmission(XL9555_ADDR);
  Wire.write(XL9555_CONFIG_PORT_0);
  Wire.write(0x00);  // 设置所有引脚为输出
  Wire.endTransmission();
  
  // 重置XL9555的所有输出为低电平
  Wire.beginTransmission(XL9555_ADDR);
  Wire.write(XL9555_OUTPUT_PORT_0);
  Wire.write(0x00);  // 所有输出设置为低
  Wire.endTransmission();
  
  // 为编码器A相添加中断
  attachInterrupt(digitalPinToInterrupt(ENCODER1_A), encoderISR1, RISING);
  attachInterrupt(digitalPinToInterrupt(ENCODER2_A), encoderISR2, RISING);
  
  Serial.println("Dual Motor Control and Encoder Speed Measurement System Started");
}

void setMotorXL9555(int motor, bool in1, bool in2) {
  uint8_t currentState;
  Wire.beginTransmission(XL9555_ADDR);
  Wire.write(XL9555_OUTPUT_PORT_0);
  Wire.endTransmission();
  Wire.requestFrom(XL9555_ADDR, 1);
  if (Wire.available()) {
    currentState = Wire.read();
  }
  
  if (motor == 1) {
    // AIN1 (IO05), AIN2 (IO04)
    currentState = (currentState & 0xCF) | (in1 << 5) | (in2 << 4);
  } else if (motor == 2) {
    // BIN1 (IO07), BIN2 (IO06)
    currentState = (currentState & 0x3F) | (in1 << 7) | (in2 << 6);
  }
  
  Wire.beginTransmission(XL9555_ADDR);
  Wire.write(XL9555_OUTPUT_PORT_0);
  Wire.write(currentState);
  Wire.endTransmission();
  
  // 调试输出
  Serial.print("Motor ");
  Serial.print(motor);
  Serial.print(" direction set to: ");
  Serial.print(in1);
  Serial.print(", ");
  Serial.println(in2);
}

void setMotor(int motor, int speed) {
  speed = constrain(speed, -255, 255);
  bool direction = (speed >= 0);
  speed = abs(speed);
  
  if (motor == 1) {
    setMotorXL9555(1, direction, !direction);
    ledcWrite(PWM_CHANNEL_A, speed);
  } else if (motor == 2) {
    setMotorXL9555(2, direction, !direction);
    ledcWrite(PWM_CHANNEL_B, speed);
  }
  
  // 调试输出
  Serial.print("Motor ");
  Serial.print(motor);
  Serial.print(" speed set to: ");
  Serial.println(speed);
}

void loop() {
  // 每100ms计算一次速度
  if (millis() - lastTime >= 100) {
    // 计算电机1速度
    long currentPosition1 = encoderValue1;
    long positionChange1 = currentPosition1 - lastPosition1;
    
    // 计算电机2速度
    long currentPosition2 = encoderValue2;
    long positionChange2 = currentPosition2 - lastPosition2;
    
    unsigned long timeChange = millis() - lastTime;
    
    // 计算RPM
    rpm1 = (float)(positionChange1 * 600.0) / (float)(PULSES_PER_REVOLUTION * (timeChange / 1000.0));
    rpm2 = (float)(positionChange2 * 600.0) / (float)(PULSES_PER_REVOLUTION * (timeChange / 1000.0));
    
    // 打印结果
    Serial.print("Motor 1 - Encoder Value: ");
    Serial.print(currentPosition1);
    Serial.print(", Speed: ");
    Serial.print(rpm1);
    Serial.print(" RPM | ");
    
    Serial.print("Motor 2 - Encoder Value: ");
    Serial.print(currentPosition2);
    Serial.print(", Speed: ");
    Serial.print(rpm2);
    Serial.println(" RPM");
    
    // 更新上次位置和时间
    lastPosition1 = currentPosition1;
    lastPosition2 = currentPosition2;
    lastTime = millis();
  }
  
  // 测试电机控制
  static unsigned long lastMotorChangeTime = 0;
  if (millis() - lastMotorChangeTime >= 5000) {  // 每5秒改变一次电机状态
    static int motorState = 0;
    switch(motorState) {
      case 0:
        Serial.println("Setting Motor 1 to -128, Motor 2 to 128");
        setMotor(1, -128);  // 电机1反转
        setMotor(2, 128);   // 电机2正转
        break;
      case 1:
        Serial.println("Setting Motor 1 to 128, Motor 2 to -128");
        setMotor(1, 128);   // 电机1正转
        setMotor(2, -128);  // 电机2反转
        break;
      case 2:
        Serial.println("Stopping both motors");
        setMotor(1, 0);
        setMotor(2, 0);
        break;
    }
    motorState = (motorState + 1) % 3;
    lastMotorChangeTime = millis();
  }
}